% FD2d_LShape
% Arendt, Urban: Partielle Differenzialgleichungen, 2. Aufl. 2018
%
%   L-shape (-1,1)^2\([0,1]x[-1,0])
%   rechte Seite f = 0
%   Rendwerte    g = entsprechend (bzgl. u)
%   -> exakte Loesung lautet 
%     u(x,y) = r^(2/3)*sin(2/3phi), 
%     mit (x,y) =r(cos(phi),sin(phi)), phi>=0
% ------------------------------------------------------------------------------

clear all;
close all;

f = @(x,y) zeros(size(x));
u = @(x,y) abs(complex(x,y)).^(2/3).*sin(2/3*mod(angle(complex(x,y)),2*pi));
[bnd(1:4).g] = deal(u,u,u,u);
[bnd(1:4).t] = deal(1,2,3,4);

NList = 20*2.^(0:5)-1;

%NList = 39;            % nur um den Fehlerplot in Abb. 9.4 zu erzeugen
hList = 1./(NList+1);   % fuer die Fehlerhistorie
errList = [];

for Np2=NList+2
  
  % Gitter
  [X,Y] = meshgrid(linspace(-1,1,Np2));
  Y = flipud(Y);
  mid = ceil(Np2/2);
  
  % G = (-1,1)^2\([0,1]x[-1,0])
  G = reshape(1:Np2^2,Np2,Np2);
  G(mid+1:end,mid+1:end) = 0;
  
  % Indizes der Punkte oben, rechts, unten und links am Rand
  bnd(1).p = G(1,:)';
  bnd(2).p = [G(1:mid,end); G(mid:Np2,mid)];
  bnd(3).p = [G(end,1:mid) G(mid,mid:Np2)]';
  bnd(4).p = G(:,1);
  
  % loese pde
  ud = solveStat(X,Y,f,bnd,G);

  % Fehler zur exakten Loesung
  ua = u(X,Y).*(G>0);
  err = abs(ud-ua);
  errList = [errList norm(err(:),inf)];
end

fprintf('h\t err\n');
for n=1:size(errList,2)
    fprintf('%15.13e\t %15.13e\n',2^(-n)*0.1,errList(n));
end

% Loesung plotten
figure(1)
set(gcf,'Color',[1 1 1]);
set(gcf,'Position',[5 30 800 600]);
set(gcf,'PaperPositionMode','auto');

ua(G==0) = NaN;
surf(X,Y,ua);
colormap gray;
shading flat;
print(gcf, '-dpdf','FD2d_LShape_Solution.pdf','-bestfit');

% Fehler anzeigen
figure(2)
set(gcf,'Color',[1 1 1]);
set(gcf,'Position',[5 30 800 600]);
set(gcf,'PaperPositionMode','auto');

err(G==0) = NaN;
mesh(X,Y,err,log10(err));
set(gca,'ZScale','log');
colormap gray;
shading flat;
grid off;
print(gcf, '-dpdf','FD2d_LShape_Error.pdf','-bestfit');


% Fehlerhistorie
figure(3)
set(gcf,'Color',[1 1 1]);
set(gcf,'Position',[5 30 800 600]);
set(gcf,'PaperPositionMode','auto');

p = polyfit(log(hList),log(errList),1);
loglog(hList,errList,'ko-');
xlabel('h');
grid on;

print(gcf, '-dpdf','FD2d_LShape_Decay_of_Errors.pdf','-bestfit');

conv_rate = mean( diff(log(errList)) ./ diff(log(hList)) );
fprintf('Estimated rate of convergence is h^p, where p = %4.3e\n',1/conv_rate);

