%   Numerisches Beispiel mit MATLAB zum expliziten Eulerverfahren fuer
%   die Wellengleichung mit f=0
%   u_t = c^2 u_xx              in [0,T]x[0,1],
%   u(t,0)=0=u(t,1)             (0<t<T),
%   u(0,x)=u0(x)                (0<x<1),
%   u_t(0,x)=u1(x)              (0<x<1).
clear all;

% Parameter
N=100;      % Anzahl Zeitschritte
M=50;     % Anzahl Gitterpunkt im Ort 
T=1;
c=0.02;
%method='first';
method='second';


% -- Test 1 --
% Anfangsdaten
% u0 = @(x) x*0;
% u1 = @(x) (2*pi*c)*sin(2*pi*x);
% 
% lsgT = @(t) sin(2*pi*c*t);
% lsgX = @(x) sin(2*pi*x);

% -- Test 2 --
% Anfangsdaten
% u0 = @(x) -(1/2).*x.*(1-x);
% u1 = @(x) (-1/2*c^2)*u0(x);
% 
% lsgT = @(t) 1+c^2*u0(t);
% lsgX = @(x) u0(x);

% -- Test 3 --
% Anfangsdaten
u0 = @(x) sin(2*pi*x);
u1 = @(x) 0*x;

lsgT = @(t) cos(2*pi*c*t);
lsgX = @(x) sin(2*pi*x);

%--------------------------------


% Konstanten
dx=1/M;
dt=T/N;
alpha=c*dt/dx;
xgrid=linspace(0,1,M+1);
tgrid=linspace(0,T,N+1);

fprintf('dx         = %6.3f \n', dx);
fprintf('dt         = %6.3f \n', dt);
fprintf('CFL: alpha = %6.3f (< 1?)\n', alpha);

% Initialisiere Tridiagonalmatrix A  mit
% A = (alpha^2, 2 - 2 alpha^2, alpha^2) 
A = spdiags([     alpha^2*ones(M-1,1), ...
            2*(1-alpha^2)*ones(M-1,1), ...
                  alpha^2*ones(M-1,1)],[-1,0,1],M-1,M-1);

            
% Anfangsbedingungen
u=u0(xgrid);
U=zeros(N+1,M+1);
U=[u];


switch lower(method)
    case {'first'}
        % Erster Zeitschritt mit niedriger Ordnung
        u = U(1,:) + dt*u1(xgrid);
        %--
    otherwise
        % Erster Zeitschritt mit hoher Ordnung
        u(1)   = 0;
        u(2:M) = 0.5 * A * U(1,2:M)' ;
        u(M+1) = 0;
        u = u + dt * u1(xgrid);
        %-- 
end


U = [U;u];


% Schleife Zeitschritte ab 2. Iterierter
for n=2:N
  u(1)=0;
  u(2:M) = A * U(n,2:M)' - U(n-1,2:M)';
  u(M+1)=0;
  U=[U;u];  
end;

% Fehlerberechnung
E = zeros(N+1,M+1);
E = (U - lsgT(tgrid)' *  lsgX(xgrid));

errors = zeros(1,N+1);
for i=1:N+1
    errors(i)=norm(E(i,:),inf);
end
figure(3); plot(tgrid,errors);

% Graphik
[nr,nc]=size(U);
%inc=floor(nr/41);
inc=1;
%--
figure(1);
mesh(0:inc*dt:T,xgrid,U([1:inc:nr],:)');
view(45,45);
title(' Numerische Approximation von u_tt=c^2 u_xx mit Expl. Euler-Verfahren');
xlabel('t');
ylabel('x');
zlabel('U(x,t)');
%--
figure(2);
mesh(0:inc*dt:T,xgrid,E([1:inc:nr],:)');
view(45,45);
title(' Fehler');
xlabel('t');
ylabel('x');
zlabel('Error(x,t)');
%--


