%   Numerisches Beispiel mit MATLAB zum impliziten Eulerverfahren fuer
%   die Wellengleichung mit f=0
%   u_t = c^2 u_xx              in [0,T]x[0,1],
%   u(t,0)=0=u(t,1)             (0<t<T),
%   u(0,x)=u0(x)                (0<x<1),
%   u_t(0,x)=u1(x)              (0<x<1).
clear all;
format long e;

% Parameter
N=100;      % Anzahl Zeitschritte
M=50;     % Anzahl Gitterpunkt im Ort 
T=1;
c=2.025;  %2.025 fuer die Experimente
theta=0.5;
%method='first';
method='second';


% -- Test 1 --
% Anfangsdaten
% u0 = @(x) x*0;
% u1 = @(x) (2*pi*c)*sin(2*pi*x);
% 
% lsgT = @(t) sin(2*pi*c*t);
% lsgX = @(x) sin(2*pi*x);

% -- Test 2 --
% Anfangsdaten
% u0 = @(x) -(1/2).*x.*(1-x);
% u1 = @(x) (-1/2*c^2)*u0(x);
% 
% lsgT = @(t) 1+c^2*u0(t);
% lsgX = @(x) u0(x);

% -- Test 3 --
% Anfangsdaten
u0 = @(x) sin(2*pi*x);
u1 = @(x) 0*x;

lsgT = @(t) cos(2*pi*c*t);
lsgX = @(x) sin(2*pi*x);

%--------------------------------


% Konstanten
dx=1/M;
dt=T/N;
kappa=dt/dx;
alpha=c*kappa;
cfl=1/(c*sqrt(1-4*theta));
xgrid=linspace(0,1,M+1);
tgrid=linspace(0,T,N+1);

fprintf('dx         = %6.3f \n', dx);
fprintf('dt         = %6.3f \n', dt);
fprintf('theta      = %6.3f \n', theta);
if (theta<0.25)
    fprintf('CFL (fuer theta<1/4): kappa = %6.3f (<= %6.3f?)\n',...
            kappa,cfl);
else
    fprintf('CFL stets erfuellt (theta>=1/4)\n');
end

% Initialisiere Tridiagonalmatrix A  mit
Lh = spdiags([      -ones(M-1,1), ...
                   2*ones(M-1,1), ...
                   -ones(M-1,1)],[-1,0,1],M-1,M-1);
Lh=(c^2/dx^2)*Lh;

A=2*eye(M-1)-(dt^2)*(1-2*theta)*Lh;
B=eye(M-1)+theta*(dt^2)*Lh;

A1=2*eye(M-1);
B1=eye(M-1)+0.5*dt^2*Lh;


% Anfangsbedingungen
u=u0(xgrid);
U=zeros(N+1,M+1);
U=[u];


switch lower(method)
    case {'first'}
        % Erster Zeitschritt mit niedriger Ordnung
        u = U(1,:) + dt*u1(xgrid);
        %--
    otherwise
        % Erster Zeitschritt mit hoher Ordnung
        u(1)   = 0;
        u(2:M) = 0.5 * A * U(1,2:M)' ;
        u(M+1) = 0;
        u = u + dt * u1(xgrid);
        %-- 
end


U = [U;u];



% Schleife Zeitschritte ab 2. Iterierter
for n=2:N
  u(1)=0;
  u(2:M) = B\(A * U(n,2:M)') - U(n-1,2:M)';
  u(M+1)=0;
  U=[U;u];  
end;

% Fehlerberechnung
E = zeros(N+1,M+1);
E = (U - lsgT(tgrid)' *  lsgX(xgrid));

errors = zeros(1,N+1);
for i=1:N+1
    errors(i)=norm(E(i,:),inf);
end
figure(3); 
plot(tgrid,errors,'k-');
set(gca,'fontsize',20)
%----------------------------------------------------
ps = get(gcf, 'Position');
ratio = (ps(4)-ps(2)) / (ps(3)-ps(1));
paperWidth = 12;
paperHeight = paperWidth*ratio;

set(gcf, 'paperunits', 'centimeters');
set(gcf, 'papersize', [paperWidth paperHeight]);
set(gcf, 'PaperPosition', [0    0   paperWidth paperHeight]);

print(gcf,'-dpdf','WaveFDMaxErrors.pdf');
%----------------------------------------------------

% Graphik
[nr,nc]=size(U);
%inc=floor(nr/41);
inc=1;
%--
figure(1);
mesh(0:inc*dt:T,xgrid,U([1:inc:nr],:)');
view(45,45);
title(' Numerische Approximation von u_tt=c^2 u_xx mit Expl. Euler-Verfahren');
xlabel('t');
ylabel('x');
zlabel('U(x,t)');
%--
figure(2);
mesh(0:inc*dt:T,xgrid,E([1:inc:nr],:)');
view(45,45);
% title(' Fehler');
% xlabel('t');
% ylabel('x');
% zlabel('Error(x,t)');
colormap(gray);

set(gca,'fontsize',20);
%----------------------------------------------------
ps = get(gcf, 'Position');
ratio = (ps(4)-ps(2)) / (ps(3)-ps(1));
paperWidth = 12;
paperHeight = paperWidth*ratio;

set(gcf, 'paperunits', 'centimeters');
set(gcf, 'papersize', [paperWidth paperHeight]);
set(gcf, 'PaperPosition', [0    0   paperWidth paperHeight]);

print(gcf,'-dpdf','WaveFDErrors.pdf');
%----------------------------------------------------



